@@ -1,22 +1,18 @@
 package Net::DNS::RR::CDS;
 
-#
-# $Id: CDS.pm 1586 2017-08-15 09:01:57Z willem $
-#
-our $VERSION = (qw$LastChangedRevision: 1586 $)[1];
-
-
 use strict;
 use warnings;
+our $VERSION = (qw$Id: CDS.pm 1857 2021-12-07 13:38:02Z willem $)[2];
+
 use base qw(Net::DNS::RR::DS);
 
+
 =head1 NAME
 
 Net::DNS::RR::CDS - DNS CDS resource record
 
 =cut
 
-
 use integer;
 
 
@@ -25,12 +21,13 @@ sub algorithm {
 	return $self->SUPER::algorithm($arg) if $arg;
 	return $self->SUPER::algorithm() unless defined $arg;
 	@{$self}{qw(keytag algorithm digtype)} = ( 0, 0, 0 );
+	return;
 }
 
 
 sub digtype {
 	my ( $self, $arg ) = @_;
-	$self->SUPER::digtype( $arg ? $arg : () );
+	return $self->SUPER::digtype( $arg ? $arg : () );
 }
 
 
@@ -48,7 +45,7 @@ __END__
 =head1 SYNOPSIS
 
     use Net::DNS;
-    $rr = new Net::DNS::RR('name CDS keytag algorithm digtype digest');
+    $rr = Net::DNS::RR->new('name CDS keytag algorithm digtype digest');
 
 =head1 DESCRIPTION
 
@@ -83,7 +80,7 @@ Package template (c)2009,2012 O.M.Kolkman and R.W.Franks.
 
 Permission to use, copy, modify, and distribute this software and its
 documentation for any purpose and without fee is hereby granted, provided
-that the above copyright notice appear in all copies and that both that
+that the original copyright notices appear in all copies and that both
 copyright notice and this permission notice appear in supporting
 documentation, and that the name of the author not be used in advertising
 or publicity pertaining to distribution of the software without specific
