@@ -1,26 +1,22 @@
 package Net::DNS::RR::NSEC;
 
-#
-# $Id: NSEC.pm 1692 2018-07-06 08:55:39Z willem $
-#
-our $VERSION = (qw$LastChangedRevision: 1692 $)[1];
-
-
 use strict;
 use warnings;
+our $VERSION = (qw$Id: NSEC.pm 1857 2021-12-07 13:38:02Z willem $)[2];
+
 use base qw(Net::DNS::RR);
 
+
 =head1 NAME
 
 Net::DNS::RR::NSEC - DNS NSEC resource record
 
 =cut
 
-
 use integer;
 
 use Net::DNS::DomainName;
-use Net::DNS::Parameters;
+use Net::DNS::Parameters qw(:type);
 
 
 sub _decode_rdata {			## decode rdata from wire-format octet string
@@ -28,8 +24,9 @@ sub _decode_rdata {			## decode rdata from wire-format octet string
 	my ( $data, $offset ) = @_;
 
 	my $limit = $offset + $self->{rdlength};
-	( $self->{nxtdname}, $offset ) = decode Net::DNS::DomainName(@_);
+	( $self->{nxtdname}, $offset ) = Net::DNS::DomainName->decode(@_);
 	$self->{typebm} = substr $$data, $offset, $limit - $offset;
+	return;
 }
 
 
@@ -37,7 +34,7 @@ sub _encode_rdata {			## encode rdata as wire-format octet string
 	my $self = shift;
 
 	my $nxtdname = $self->{nxtdname};
-	join '', $nxtdname->encode(), $self->{typebm};
+	return join '', $nxtdname->encode(), $self->{typebm};
 }
 
 
@@ -45,7 +42,7 @@ sub _format_rdata {			## format rdata portion of RR string.
 	my $self = shift;
 
 	my $nxtdname = $self->{nxtdname};
-	my @rdata = ( $nxtdname->string(), $self->typelist );
+	return ( $nxtdname->string(), $self->typelist );
 }
 
 
@@ -54,23 +51,35 @@ sub _parse_rdata {			## populate RR from rdata in argument list
 
 	$self->nxtdname(shift);
 	$self->typelist(@_);
+	return;
+}
+
+
+sub _defaults {				## specify RR attribute default values
+	my $self = shift;
+
+	$self->_parse_rdata('.');
+	return;
 }
 
 
 sub nxtdname {
 	my $self = shift;
 
-	$self->{nxtdname} = new Net::DNS::DomainName(shift) if scalar @_;
-	$self->{nxtdname}->name if $self->{nxtdname};
+	$self->{nxtdname} = Net::DNS::DomainName->new(shift) if scalar @_;
+	return $self->{nxtdname} ? $self->{nxtdname}->name : undef;
 }
 
 
 sub typelist {
 	my $self = shift;
 
-	$self->{typebm} = &_type2bm if scalar @_;
+	if ( scalar(@_) || !defined(wantarray) ) {
+		$self->{typebm} = &_type2bm;
+		return;
+	}
 
-	my @type = defined wantarray ? &_bm2type( $self->{typebm} ) : ();
+	my @type = &_bm2type( $self->{typebm} );
 	return wantarray ? (@type) : "@type";
 }
 
@@ -96,6 +105,13 @@ sub typemap {
 }
 
 
+sub match {
+	my $self = shift;
+	my $name = Net::DNS::DomainName->new(shift)->canonical;
+	return $name eq $self->{owner}->canonical;
+}
+
+
 sub covers {
 	my $self = shift;
 	my $name = join chr(0), reverse Net::DNS::DomainName->new(shift)->_wire;
@@ -108,11 +124,38 @@ sub covers {
 }
 
 
+sub encloser {
+	my $self  = shift;
+	my @qname = Net::DNS::Domain->new(shift)->label;
+
+	my @owner = $self->{owner}->label;
+	my $depth = scalar(@owner);
+	my $next;
+	while ( scalar(@qname) > $depth ) {
+		$next = shift @qname;
+	}
+
+	return unless defined $next;
+
+	my $nextcloser = join( '.', $next, @qname );
+	return if lc($nextcloser) ne lc( join '.', $next, @owner );
+
+	$self->{nextcloser} = $nextcloser;
+	$self->{wildcard}   = join( '.', '*', @qname );
+	return $self->owner;
+}
+
+
+sub nextcloser { return shift->{nextcloser}; }
+
+sub wildcard { return shift->{wildcard}; }
+
+
 ########################################
 
 sub _type2bm {
 	my @typearray;
-	foreach my $typename ( map split(), @_ ) {
+	foreach my $typename ( map { split() } @_ ) {
 		my $number = typebyname($typename);
 		my $window = $number >> 8;
 		my $bitnum = $number & 255;
@@ -125,7 +168,7 @@ sub _type2bm {
 	my $window = 0;
 	foreach (@typearray) {
 		if ( my $pane = $typearray[$window] ) {
-			my @content = map $_ || 0, @$pane;
+			my @content = map { $_ || 0 } @$pane;
 			$bitmap .= pack 'CC C*', $window, scalar(@content), @content;
 		}
 		$window++;
@@ -165,9 +208,17 @@ sub _bm2type {
 sub typebm {				## historical
 	my $self = shift;					# uncoverable pod
 	$self->{typebm} = shift if scalar @_;
+	$self->_deprecate('prefer $rr->typelist() or $rr->typemap()');
 	return $self->{typebm};
 }
 
+sub covered {				## historical
+	my $self = shift;					# uncoverable pod
+	return $self->covers(@_);
+}
+
+########################################
+
 
 1;
 __END__
@@ -176,7 +227,7 @@ __END__
 =head1 SYNOPSIS
 
     use Net::DNS;
-    $rr = new Net::DNS::RR( 'name NSEC nxtdname typelist' );
+    $rr = Net::DNS::RR->new( 'name NSEC nxtdname typelist' );
 
 =head1 DESCRIPTION
 
@@ -217,6 +268,14 @@ into a string.
 typemap() returns a Boolean true value if the specified RRtype occurs
 in the type bitmap of the NSEC record.
 
+=head2 match
+
+    $matched = $rr->match( 'example.foo' );
+
+match() returns a Boolean true value if the canonical form of the name
+argument matches the canonical owner name of the NSEC RR.
+
+
 =head2 covers
 
     $covered = $rr->covers( 'example.foo' );
@@ -225,12 +284,28 @@ covers() returns a Boolean true value if the canonical form of the name,
 or one of its ancestors, falls between the owner name and the nxtdname
 field of the NSEC record.
 
+=head2 encloser, nextcloser, wildcard
+
+    $encloser = $rr->encloser( 'example.foo' );
+    print "encloser: $encloser\n" if $encloser;
+
+encloser() returns the name of a provable encloser of the query name
+argument obtained from the NSEC RR.
+
+nextcloser() returns the next closer name, which is one label longer
+than the closest encloser.
+This is only valid after encloser() has returned a valid domain name.
+
+wildcard() returns the unexpanded wildcard name from which the next
+closer name was possibly synthesised.
+This is only valid after encloser() has returned a valid domain name.
+
 
 =head1 COPYRIGHT
 
 Copyright (c)2001-2005 RIPE NCC.  Author Olaf M. Kolkman
 
-Portions Copyright (c)2018 Dick Franks
+Portions Copyright (c)2018-2019 Dick Franks
 
 All rights reserved.
 
@@ -241,7 +316,7 @@ Package template (c)2009,2012 O.M.Kolkman and R.W.Franks.
 
 Permission to use, copy, modify, and distribute this software and its
 documentation for any purpose and without fee is hereby granted, provided
-that the above copyright notice appear in all copies and that both that
+that the original copyright notices appear in all copies and that both
 copyright notice and this permission notice appear in supporting
 documentation, and that the name of the author not be used in advertising
 or publicity pertaining to distribution of the software without specific
@@ -258,6 +333,6 @@ DEALINGS IN THE SOFTWARE.
 
 =head1 SEE ALSO
 
-L<perl>, L<Net::DNS>, L<Net::DNS::RR>, RFC4034, RFC3755
+L<perl>, L<Net::DNS>, L<Net::DNS::RR>, RFC4034, RFC9077
 
 =cut
